;; Generated package description from bnf-mode.el
(define-package "bnf-mode" "0.4.2" "Major mode for editing BNF grammars." '((cl-lib "0.5") (emacs "24.3")) :url "https://github.com/sergeyklay/bnf-mode" :keywords '("languages"))
